package cn.iocoder.yudao.module.system.service.remote;

import cn.hutool.core.util.ObjectUtil;
import cn.iocoder.yudao.framework.common.util.servlet.ServletUtils;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.system.api.remote.dto.LoginCheckDTO;
import cn.iocoder.yudao.module.system.api.remote.dto.RemoteRobotStatusDTO;
import cn.iocoder.yudao.module.system.controller.admin.log.vo.UserOperationLogSaveReqVO;
import cn.iocoder.yudao.module.system.controller.admin.remote.vo.RemoteControllerInformationPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.remote.vo.RemoteControllerInformationSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.log.RobotTaskDetailActionLogDO;
import cn.iocoder.yudao.module.system.dal.dataobject.remote.RemoteControllerInformationDO;
import cn.iocoder.yudao.module.system.dal.dataobject.robot.RobotInformationDO;
import cn.iocoder.yudao.module.system.dal.dataobject.robot.RobotMapStopDO;
import cn.iocoder.yudao.module.system.dal.mysql.remote.RemoteControllerInformationMapper;
import cn.iocoder.yudao.module.system.enums.common.ZeroOneEnum;
import cn.iocoder.yudao.module.system.enums.path.PathTaskTypeEnum;
import cn.iocoder.yudao.module.system.enums.robot.actionlog.ActionStatusEnum;
import cn.iocoder.yudao.module.system.service.log.RobotTaskDetailActionLogService;
import cn.iocoder.yudao.module.system.service.log.UserOperationLogService;
import cn.iocoder.yudao.module.system.service.robot.mapstop.RobotMapStopService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

import org.springframework.validation.annotation.Validated;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;


import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.CONTROLLER_INFORMATION_NOT_EXISTS;

/**
 * 远遥设备信息 Service 实现类
 *
 * @author 陈宾顺
 */
@Service
@Validated
public class RemoteControllerInformationServiceImpl extends ServiceImpl<RemoteControllerInformationMapper, RemoteControllerInformationDO> implements RemoteControllerInformationService {

    @Resource
    private RemoteControllerInformationMapper controllerInformationMapper;

    @Resource
    private UserOperationLogService userOperationLogService;

    @Resource
    private RobotMapStopService mapStopService;

    @Resource
    private RobotTaskDetailActionLogService taskDetailActionLogService;

    @Override
    public Long createControllerInformation(RemoteControllerInformationSaveReqVO createReqVO) {
        // 插入
        RemoteControllerInformationDO controllerInformation = BeanUtils.toBean(createReqVO, RemoteControllerInformationDO.class);
        controllerInformationMapper.insert(controllerInformation);
        // 返回
        return controllerInformation.getId();
    }

    @Override
    public void updateControllerInformation(RemoteControllerInformationSaveReqVO updateReqVO) {
        // 校验存在
        validateControllerInformationExists(updateReqVO.getId());
        // 更新
        RemoteControllerInformationDO updateObj = BeanUtils.toBean(updateReqVO, RemoteControllerInformationDO.class);
        controllerInformationMapper.updateById(updateObj);
    }

    @Override
    public void deleteControllerInformation(Long id) {
        // 校验存在
        validateControllerInformationExists(id);
        // 删除
        controllerInformationMapper.deleteById(id);
    }

    private void validateControllerInformationExists(Long id) {
        if (controllerInformationMapper.selectById(id) == null) {
            throw exception(CONTROLLER_INFORMATION_NOT_EXISTS);
        }
    }

    @Override
    public RemoteControllerInformationDO getControllerInformation(Long id) {
        return controllerInformationMapper.selectById(id);
    }

    @Override
    public PageResult<RemoteControllerInformationDO> getControllerInformationPage(RemoteControllerInformationPageReqVO pageReqVO) {
        return controllerInformationMapper.selectPage(pageReqVO);
    }

    /**
     * 保存IP和端口
     *
     * @param loginCheck
     */
    @Override
    public void saveIpAndPort(LoginCheckDTO loginCheck) {
        RemoteControllerInformationDO information = controllerInformationMapper.selectOne(new LambdaQueryWrapper<RemoteControllerInformationDO>()
                .eq(RemoteControllerInformationDO::getRemoteIp, loginCheck.getRemoteIp())
                .last("limit 1"));
        UserOperationLogSaveReqVO operationLog = null;
        if (ObjectUtil.isEmpty(information)) {
            RemoteControllerInformationDO build = RemoteControllerInformationDO.builder()
                    .remoteIp(loginCheck.getRemoteIp())
                    .systemPort(loginCheck.getSystemPort())
                    .systemIp(loginCheck.getSystemIp())
                    .build();
            controllerInformationMapper.insert(build);

            operationLog = UserOperationLogSaveReqVO.builder()
                    .operateAction("添加远遥设备IP " + loginCheck.getSystemIp() + " 端口 " + loginCheck.getSystemPort())
                    .nickName(SecurityFrameworkUtils.getLoginUserNickname()).build();

        } else {
            information.setSystemIp(loginCheck.getSystemIp());
            information.setSystemPort(loginCheck.getSystemPort());
            controllerInformationMapper.updateById(information);

            operationLog = UserOperationLogSaveReqVO.builder()
                    .operateAction("修改远遥设备IP " + loginCheck.getSystemIp() + " 端口 " + loginCheck.getSystemPort())
                    .nickName(SecurityFrameworkUtils.getLoginUserNickname()).build();
        }

        userOperationLogService.createUserOperationLog(operationLog);
    }

    /**
     * 查询远遥默认IP和端口
     * @return
     */
    @Override
    public LoginCheckDTO getDefaultIpAndPort(String remoteIp) {
        RemoteControllerInformationDO information = controllerInformationMapper.selectOne(new LambdaQueryWrapper<RemoteControllerInformationDO>()
                .eq(RemoteControllerInformationDO::getRemoteIp, remoteIp)
                .last("limit 1"));
        LoginCheckDTO loginCheck = new LoginCheckDTO();
        if (ObjectUtil.isEmpty(information)) {
            return loginCheck;
        }
        loginCheck.setSystemIp(information.getSystemIp());
        loginCheck.setSystemPort(information.getSystemPort());
        return loginCheck;
    }

    @Override
    public List<RemoteControllerInformationDO> getAllRemoteController() {
        return controllerInformationMapper.selectList(new LambdaQueryWrapper<RemoteControllerInformationDO>()
                .ne(RemoteControllerInformationDO::getRobotNo, ""));
    }

    /**
     * 获取机器人远遥模式/急停/协控状态
     * @param remoteIp
     * @return
     */
    @Override
    public RemoteRobotStatusDTO getRemoteRobotStatus(String remoteIp) {
        RemoteControllerInformationDO information = controllerInformationMapper.selectOne(new LambdaQueryWrapper<RemoteControllerInformationDO>()
                .eq(RemoteControllerInformationDO::getRemoteIp, remoteIp)
                .last("limit 1"));
        if (ObjectUtil.isEmpty(information) || ObjectUtil.isEmpty(information.getRobotNo())) {
            return null;
        }
        RemoteRobotStatusDTO remoteRobotStatus = BeanUtils.toBean(information, RemoteRobotStatusDTO.class);
        List<RobotMapStopDO> robotMapStops = mapStopService.getStopRobotNoByRobotNo(information.getRobotNo());
        if (ObjectUtil.isNotEmpty(robotMapStops)) {
            remoteRobotStatus.setIsStop(ZeroOneEnum.ZERO.getType());
        }
        RobotTaskDetailActionLogDO actionLog = taskDetailActionLogService.getLastTaskByRobotNo(information.getRobotNo());
        if (ObjectUtil.isNotEmpty(actionLog)  && !actionLog.getActionStatus().equals(ActionStatusEnum.DONE.getType())
            && (PathTaskTypeEnum.TAKE_RELEASE.getType().equals(actionLog.getCommandType())
                || PathTaskTypeEnum.TAKE.getType().equals(actionLog.getCommandType())
                || PathTaskTypeEnum.RELEASE.getType().equals(actionLog.getCommandType())) ) {
            remoteRobotStatus.setClickTaskDone(ZeroOneEnum.ZERO.getType());
        }

        return remoteRobotStatus;
    }

    /**
     * 根据车辆编号查询
     * @param robotNo
     * @return
     */
    @Override
    public RemoteControllerInformationDO getRemoteControllerByRobotNo(String robotNo) {
        RemoteControllerInformationDO information = controllerInformationMapper.selectOne(new LambdaQueryWrapper<RemoteControllerInformationDO>()
                .eq(RemoteControllerInformationDO::getRobotNo, robotNo)
                .last("limit 1"));
        return information;
    }


}
